#include "bvp.hh"
#include <iomanip>

using namespace std ;

/*
  y'' + y' - 3*y = 9*exp(3*x) -6

  y(0) = 3
  y(1) = exp(3)+2

  // exact solution
  y(x) = exp(3*x) + 2 

 */

class my_BVP : public BVP {
public:
  virtual value_type p( value_type x ) const { return 1 ; }
  virtual value_type q( value_type x ) const { return -3 ; }
  virtual value_type r( value_type x ) const { return 9*exp(3*x) - 6 ; }
  
  value_type exact( value_type x ) const { return exp(3*x) + 2 ; }

  void
  solve( value_type a, value_type b, int N ) 
  { BVP::solve( a, exact(a), b, exact(b), N ) ; }
 
};

int
main() {
  my_BVP bvp ;
  bvp.solve( 0, 1, 100 ) ;
  vector<BVP::value_type> const & x = bvp.get_x() ;
  vector<BVP::value_type> const & y = bvp.get_y() ;
  cout << setw(4) << "index"
       << " " << setw(12) << "x" 
       << " " << setw(12) << "y"
       << " " << setw(12) << "esatta"
       << '\n' ;
  for ( int i = 0 ; i <= 100 ; ++i )
    cout << setw(4) << i
         << " " << setw(12) << x[i] 
         << " " << setw(12) << y[i]
         << " " << setw(12) << bvp.exact(x[i])
         << '\n' ;
  return 0 ;
}

