/*

  Inheritance and usage of it

*/

#include <iostream>
#include <iomanip>
#include <vector>
#include <cmath>

class Problem {
public:
  Problem( ) { std::cout << "call constuctor Problem class\n" ; }
  virtual double f( double x ) const = 0 ;
  virtual double df( double x ) const = 0 ;
} ;

class Problem1 : public Problem {
  double a ;
public:
  Problem1( ) : a(0) { std::cout << "call constuctor Problem1\n" ; }
  Problem1( double _a ) : a(_a) { std::cout << "call constuctor Problem1\n" ; }
  
  virtual double f( double x ) const { return x*x-a ; }
  virtual double df( double x ) const { return 2*x ; }

} ;

class Problem2 : public Problem {
public:
  Problem2( ) { std::cout << "call constuctor 1 Problem2 \n" ; }

  virtual double f( double x ) const { return exp(x)-3 ; }
  virtual double df( double x ) const { return exp(x) ; }

} ;

// Newton--Raphson
bool
Newton( Problem * pb,
        double  & x, // x is the guess and at exit the approximate solution
        int       max_iter,
        double    tol ) {
  for ( int i = 0 ; i < max_iter ; ++i ) {
    double h = pb->f(x)/pb->df(x) ;
    x -= h ;
    if ( std::abs(h) < tol ) return true ;
  }
  return false ;
}

int
main() {
  Problem1 pb1(2) ;
  Problem2 pb2 ;
  double x = 2 ;
  bool ok = Newton( &pb1, x, 10, 1E-10 ) ;
  if ( ok ) {
    std::cout << "Newton converged, x = " << x << '\n' ;
  } else {
    std::cout << "Newton failed, x = " << x << '\n' ;
  }
  x = 2 ;
  ok = Newton( &pb2, x, 10, 1E-10 ) ;
  if ( ok ) {
    std::cout << "Newton converged, x = " << x << '\n' ;
  } else {
    std::cout << "Newton failed, x = " << x << '\n' ;
  }
  return 0 ;
}

